@@ -28,17 +28,16 @@
 struct gdnsd_fmap_s_;
 typedef struct gdnsd_fmap_s_ gdnsd_fmap_t;
 
-#pragma GCC visibility push(default)
-
-// Given a filename "fn", this will open the file for reading with an fcntl
-//   advisory readlock and mmap() it for readonly use.
-// On such errors, the return value is NULL.
+// Given a filename "fn", this will open the file for reading
+//   and mmap() it for readonly use.
+// On errors, the return value is NULL.
 // This can succeed for zero-length files.  In that case the accessors below
 //   will return length 0 and a valid pointer to 1 NUL byte.
 // "seq" is an optimization hint: set to true if expected buffer access
 //   pattern is sequential.
-F_NONNULL F_MALLOC
-gdnsd_fmap_t* gdnsd_fmap_new(const char* fn, const bool seq);
+// "mod" gives a writeable private buffer, instead of a readonly shared one
+F_NONNULL F_WUNUSED
+gdnsd_fmap_t* gdnsd_fmap_new(const char* fn, const bool seq, const bool mod);
 
 // Get the length of the mapped file data (zero is possible)
 F_NONNULL F_PURE
@@ -46,17 +45,14 @@ size_t gdnsd_fmap_get_len(const gdnsd_fmap_t* fmap);
 
 // Get the buffer pointer for the mapped file data (always a valid pointer)
 F_NONNULL F_RETNN F_PURE
-const void* gdnsd_fmap_get_buf(const gdnsd_fmap_t* fmap);
+void* gdnsd_fmap_get_buf(const gdnsd_fmap_t* fmap);
 
 // Destructs the fmap_t object, which includes unmap() of the memory
-//   returned via fmap_get_buf() and closing the file descriptor, which
-//   implicitly also releases the fcntl advisory readlock.
+//   returned via fmap_get_buf().
 // If a destruction step fails, this returns true (in which case the file data
 //   should perhaps be considered suspect, even if the caller managed to
 //   operate on it without error).
 F_NONNULL
 bool gdnsd_fmap_delete(gdnsd_fmap_t* fmap);
 
-#pragma GCC visibility pop
-
 #endif // GDNSD_FILE_H
