@@ -21,9 +21,12 @@
 #define NTREE_H
 
 #include <gdnsd/compiler.h>
-#include <gdnsd/plugapi.h>
+#include <gdnsd/log.h>
+#include <gdnsd/net.h>
 
 #include <inttypes.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
 /***************************************
  * ntree_t and related methods
@@ -32,54 +35,61 @@
 // ipv6 is a uint8_t[16]
 // bit is 0->127 (MSB -> LSB)
 F_NONNULL F_UNUSED
-static void SETBIT_v6(uint8_t* ipv6, const unsigned bit) {
-    dmn_assert(bit < 128);
+static void SETBIT_v6(uint8_t* ipv6, const unsigned bit)
+{
+    gdnsd_assert(bit < 128);
     ipv6[bit >> 3] |= (1UL << (~bit & 7));
 }
 
 // Some constant IPv6 address fragments...
 
 // 96-bit prefix
-static const uint8_t start_v4compat[16] =
-    { 0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00 };
+static const uint8_t start_v4compat[16] = {
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00
+};
 
 // 96-bit prefix
-static const uint8_t start_v4mapped[16] =
-    { 0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0xFF, 0xFF,
-      0x00, 0x00, 0x00, 0x00 };
+static const uint8_t start_v4mapped[16] = {
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0xFF, 0xFF,
+    0x00, 0x00, 0x00, 0x00
+};
 
 // 96-bit prefix
-static const uint8_t start_siit[16] =
-    { 0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0xFF, 0xFF, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00 };
+static const uint8_t start_siit[16] = {
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0xFF, 0xFF, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00
+};
 
 // 96-bit prefix
-static const uint8_t start_wkp[16] =
-    { 0x00, 0x64, 0xFF, 0x9B,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00 };
+static const uint8_t start_wkp[16] = {
+    0x00, 0x64, 0xFF, 0x9B,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00
+};
 
 // 16-bit prefix
-static const uint8_t start_6to4[16] =
-    { 0x20, 0x02, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00 };
+static const uint8_t start_6to4[16] = {
+    0x20, 0x02, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00
+};
 
 // 32-bit prefix
-static const uint8_t start_teredo[16] =
-    { 0x20, 0x01, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00,
-      0x00, 0x00, 0x00, 0x00 };
+static const uint8_t start_teredo[16] = {
+    0x20, 0x01, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00
+};
 
 // zero-initializer for IPv6
 static const struct in6_addr ip6_zero = { .s6_addr = { 0 } };
@@ -103,10 +113,10 @@ static const struct in6_addr ip6_zero = { .s6_addr = { 0 } };
  *     a node index for recursion.
  */
 
-#define NN_UNDEF UINT32_MAX // special undefined dclist, never
-                            //   the result of a lookup, used to
-                            //   to get netmasks correct adjacent
-                            //   to undefined v4-like spaces...
+// special undefined dclist, never the result of a lookup, used to to get
+// netmasks correct adjacent to undefined v4-like spaces...
+#define NN_UNDEF UINT32_MAX
+
 #define NN_IS_DCLIST(x) ((x) & (1U << 31U))
 #define NN_GET_DCLIST(x) ((x) & ~(1U << 31U)) // strips high bit
 #define NN_SET_DCLIST(x) ((x) | (1U << 31U)) // sets high bit
@@ -120,11 +130,10 @@ typedef struct {
     nnode_t* store;
     unsigned ipv4;  // cached ipv4 lookup hint
     unsigned count; // raw nodes, including interior ones
-    unsigned alloc; // current allocation of store during construction,
-                    //   set to zero after _finish()
+    unsigned alloc; // current allocation of store during construction, set to zero after _finish()
 } ntree_t;
 
-F_WUNUSED
+F_WUNUSED F_RETNN
 ntree_t* ntree_new(void);
 
 F_NONNULL
