@@ -12,13 +12,13 @@ fi
 set -x
 set -e
 
-CFLAGS="-O0 -g -fprofile-arcs -ftest-coverage" CPPFLAGS="-DDMN_NO_UNREACH_BUILTIN -DDMN_NO_FATAL_COVERAGE -DDMN_COVERTEST_EXIT" ./configure --disable-developer --without-hardening
+CFLAGS="-O0 -g -fprofile-arcs -ftest-coverage" CPPFLAGS="-DGDNSD_NO_UNREACH_BUILTIN -DGDNSD_NO_FATAL_COVERAGE -DGDNSD_COVERTEST_EXIT" ./configure --disable-developer --without-hardening
 make clean
 make all
 
 lcov -c -i -d . -o gdnsd-base.info --rc lcov_branch_coverage=1
 
-make check
+SLOW_TESTS=1 make check
 
 lcov -c -d . -o gdnsd-test.info --rc lcov_branch_coverage=1
 lcov -a gdnsd-base.info -a gdnsd-test.info -o gdnsd-cov.info --rc lcov_branch_coverage=1
